
/******************************************************************************
 * This file is part of the Gluon project
 * Copyright (C) year  name of author
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "centreselector.h"
#include <QVBoxLayout>
#include <QSqlQueryModel>
#include <QToolButton>
#include <QSqlQuery>
#include <QDebug>
#include <QSqlRecord>
#include <QLabel>
#include <QRegExp>
#include <QMenu>
#include <QProgressBar>
#include <QModelIndex>
#include <QMap>


CentreSelector::CentreSelector(QWidget *parent) :
        QWidget(parent)
{
    mCurrentModelId = -1;
    QVBoxLayout * mainLayout  = new QVBoxLayout;
    mTreeWidget = new QTreeWidget(this);
    mLineEdit = new QLineEdit(this);
    mActionOrderByProtocole = new QAction(QIcon(":icons/protocole.png"),"par protocoles",this);
    mActionOrderByEtablissement= new QAction(QIcon(":icons/etablissement.png"),"par etablissement",this);
    mActionOrderByReseau= new QAction(QIcon(":icons/reseau.png"),"par reseau",this);
    
    mActionOrderByProtocole->setData("protocole");
    mActionOrderByEtablissement->setData("etablissement");
    mActionOrderByReseau->setData("reseau");
    
    mOrderButton = new QToolButton;
    mOrderButton->setIcon(QIcon(":icons/filter.png"));
    
    QMenu * orderMenu = new QMenu;
    orderMenu->addAction(mActionOrderByEtablissement);
    orderMenu->addAction(mActionOrderByProtocole);
    orderMenu->addAction(mActionOrderByReseau);
    mOrderButton->setMenu(orderMenu);
    mOrderButton->setPopupMode(QToolButton::InstantPopup);
    mOrderButton->setDefaultAction(mActionOrderByEtablissement);

    
    
    mTreeWidget->setColumnCount(1);
    QStringList header;
    header<<"centres";
    mTreeWidget->setHeaderLabels(header);
    mTreeWidget->setAlternatingRowColors(true);
    QToolButton * reloadBtn = new QToolButton;
    reloadBtn->setIcon(QIcon(":/icons/refresh.png"));
    
    
    QHBoxLayout * hLayout = new QHBoxLayout;
    hLayout->addWidget(mLineEdit);
    hLayout->addWidget(reloadBtn);
    hLayout->addWidget(mOrderButton);
    
    mainLayout->addLayout(hLayout);
    mainLayout->addWidget(mTreeWidget);
    setLayout(mainLayout);
    
    connect(orderMenu,SIGNAL(triggered(QAction*)),mOrderButton,SLOT(setDefaultAction(QAction*)));
    connect(mOrderButton,SIGNAL(pressed()),this,SLOT(loadCentre()));
    
    connect(reloadBtn,SIGNAL(clicked()),this,SLOT(loadCentre()));
    connect(mLineEdit,SIGNAL(textChanged(QString)),this,SLOT(filter(QString)));
    connect(mTreeWidget,SIGNAL(clicked(QModelIndex)),this,SLOT(itemClicked(QModelIndex)));
    loadCentre();
    
    
}
void CentreSelector::loadCentre()
{
    
    if (!QSqlDatabase::database().isOpen())
        return;
    
    mTreeWidget->clear();
    
    QSqlQuery query("SELECT c.id, c.numero, c.etat, p.nom as 'protocole', e.nom as 'etablissement', n.nom as 'network'"
                    " FROM centres c"
                    " LEFT JOIN protocoles p ON c.protocole_id = p.id"
                    " LEFT JOIN etablissements e ON c.etablissement_id = e.id"
                    " LEFT JOIN networks n ON c.network_id = n.id");
    
    
    
    QString order = mOrderButton->defaultAction()->data().toString();
    QMap <QString,QTreeWidgetItem*> categorieItem;
    QFont catFont;
    catFont.setBold(true);

    while (query.next()) {
        QString categorie = query.record().value(order).toString();
        if ( categorie.isEmpty())
            categorie="autres";
        if (!categorieItem.contains(categorie))
        {
            QTreeWidgetItem * item = new QTreeWidgetItem;
            item->setText(0,categorie);
            item->setFont(0,catFont);
            item->setIcon(0,mOrderButton->defaultAction()->icon());
            
            categorieItem[categorie] =item;
            mTreeWidget->addTopLevelItem(item);
        }
        
        QTreeWidgetItem * centreItem = new QTreeWidgetItem;
        centreItem->setText(0,query.record().value("numero").toString());
        centreItem->setData(0,Qt::UserRole,query.record().value("id").toInt());
        centreItem->setIcon(0,QIcon(":icons/centre.png"));
        categorieItem[categorie]->addChild(centreItem);
    }
}



void CentreSelector::filter(const QString &text)
{
    
    for ( int i=0; i<mTreeWidget->topLevelItemCount(); ++i)
        mTreeWidget->topLevelItem(i)->setHidden(true);
    
    foreach ( QTreeWidgetItem * item, mTreeWidget->findItems (text,Qt::MatchContains))
    {
        item->setHidden(false);
    }
    
    
}
void CentreSelector::itemClicked(QModelIndex index)
{
    if (index.data(Qt::UserRole).isNull())
    {
        mCurrentModelId = -1;
        return;
    }
    else{
        mCurrentModelId= index.data(Qt::UserRole).toInt();
        emit modelIdChange(index.data(Qt::UserRole).toInt());
    }
}

QTreeWidgetItem * CentreSelector::itemSelected()
{
    
    return mTreeWidget->selectedItems().at(0);
    
    
}
void CentreSelector::setSelected(int centreId)
{

    for ( int i=0; i<mTreeWidget->topLevelItemCount(); ++i)
    {
        QTreeWidgetItem * cat = mTreeWidget->topLevelItem(i);
        for ( int j = 0; j < cat->childCount(); ++j)
        {
            if ( cat->child(j)->data(0,Qt::UserRole).toInt() == centreId)
            {
                cat->setExpanded(true);
                cat->child(j)->setSelected(true);
                return;
            }

        }



    }

}
